@@ -37,6 +37,12 @@
  * Since: 2.40
  **/
 
+static void soup_multipart_input_stream_pollable_init (GPollableInputStreamInterface *pollable_interface, gpointer interface_data);
+
+G_DEFINE_TYPE_WITH_CODE (SoupMultipartInputStream, soup_multipart_input_stream, G_TYPE_FILTER_INPUT_STREAM,
+			 G_IMPLEMENT_INTERFACE (G_TYPE_POLLABLE_INPUT_STREAM,
+						soup_multipart_input_stream_pollable_init))
+
 enum {
 	PROP_0,
 
@@ -59,13 +65,6 @@ struct _SoupMultipartInputStreamPrivate {
 	goffset		        remaining_bytes;
 };
 
-static void soup_multipart_input_stream_pollable_init (GPollableInputStreamInterface *pollable_interface, gpointer interface_data);
-
-G_DEFINE_TYPE_WITH_CODE (SoupMultipartInputStream, soup_multipart_input_stream, G_TYPE_FILTER_INPUT_STREAM,
-                         G_ADD_PRIVATE (SoupMultipartInputStream)
-			 G_IMPLEMENT_INTERFACE (G_TYPE_POLLABLE_INPUT_STREAM,
-						soup_multipart_input_stream_pollable_init))
-
 static void
 soup_multipart_input_stream_dispose (GObject *object)
 {
@@ -202,7 +201,9 @@ static void
 soup_multipart_input_stream_init (SoupMultipartInputStream *multipart)
 {
 	SoupMultipartInputStreamPrivate *priv;
-	priv = multipart->priv = soup_multipart_input_stream_get_instance_private (multipart);
+	priv = multipart->priv = G_TYPE_INSTANCE_GET_PRIVATE (multipart,
+							      SOUP_TYPE_MULTIPART_INPUT_STREAM,
+							      SoupMultipartInputStreamPrivate);
 
 	priv->meta_buf = g_byte_array_sized_new (RESPONSE_BLOCK_SIZE);
 	priv->done_with_part = FALSE;
@@ -289,6 +290,8 @@ soup_multipart_input_stream_class_init (SoupMultipartInputStreamClass *multipart
 	GInputStreamClass *input_stream_class =
 		G_INPUT_STREAM_CLASS (multipart_class);
 
+	g_type_class_add_private (multipart_class, sizeof (SoupMultipartInputStreamPrivate));
+
 	object_class->dispose = soup_multipart_input_stream_dispose;
 	object_class->finalize = soup_multipart_input_stream_finalize;
 	object_class->constructed = soup_multipart_input_stream_constructed;
@@ -303,8 +306,7 @@ soup_multipart_input_stream_class_init (SoupMultipartInputStreamClass *multipart
 				     "Message",
 				     "The SoupMessage",
 				     SOUP_TYPE_MESSAGE,
-				     G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY |
-				     G_PARAM_STATIC_STRINGS));
+				     G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
 
 }
 
@@ -455,8 +457,8 @@ soup_multipart_input_stream_new (SoupMessage  *msg,
  * the part; a new call to this function should be done at that point,
  * to obtain the next part.
  *
- * Return value: (nullable) (transfer full): a new #GInputStream, or
- * %NULL if there are no more parts
+ * Return value: (transfer full): a new #GInputStream, or %NULL if
+ * there are no more parts
  *
  * Since: 2.40
  */
@@ -481,10 +483,9 @@ soup_multipart_input_stream_next_part (SoupMultipartInputStream  *multipart,
 }
 
 static void
-soup_multipart_input_stream_next_part_thread (GTask        *task,
-					      gpointer      object,
-					      gpointer      task_data,
-					      GCancellable *cancellable)
+soup_multipart_input_stream_next_part_thread (GSimpleAsyncResult *simple,
+					      GObject            *object,
+					      GCancellable       *cancellable)
 {
 	SoupMultipartInputStream *multipart = SOUP_MULTIPART_INPUT_STREAM (object);
 	GError *error = NULL;
@@ -494,10 +495,11 @@ soup_multipart_input_stream_next_part_thread (GTask        *task,
 
 	g_input_stream_clear_pending (G_INPUT_STREAM (multipart));
 
-	if (error)
-		g_task_return_error (task, error);
-	else
-		g_task_return_pointer (task, new_stream, g_object_unref);
+	if (g_simple_async_result_propagate_error (simple, &error))
+		return;
+
+	if (new_stream)
+		g_simple_async_result_set_op_res_gpointer (simple, new_stream, g_object_unref);
 }
 
 /**
@@ -506,51 +508,55 @@ soup_multipart_input_stream_next_part_thread (GTask        *task,
  * @io_priority: the I/O priority for the request.
  * @cancellable: a #GCancellable.
  * @callback: callback to call when request is satisfied.
- * @data: data for @callback
  *
  * Obtains a #GInputStream for the next request. See
  * soup_multipart_input_stream_next_part() for details on the
  * workflow.
  *
+ * Return value: a new #GInputStream, or %NULL if there are no more
+ * parts
+ *
  * Since: 2.40
  */
 void
 soup_multipart_input_stream_next_part_async (SoupMultipartInputStream *multipart,
-					     int                       io_priority,
+					     int                      io_priority,
 					     GCancellable	      *cancellable,
 					     GAsyncReadyCallback       callback,
 					     gpointer		       data)
 {
 	GInputStream *stream = G_INPUT_STREAM (multipart);
-	GTask *task;
+	GSimpleAsyncResult *simple;
 	GError *error = NULL;
 
 	g_return_if_fail (SOUP_IS_MULTIPART_INPUT_STREAM (multipart));
 
-	task = g_task_new (multipart, cancellable, callback, data);
-	g_task_set_priority (task, io_priority);
+	simple = g_simple_async_result_new (G_OBJECT (multipart),
+					    callback, data,
+					    soup_multipart_input_stream_next_part_async);
 
 	if (!g_input_stream_set_pending (stream, &error)) {
-		g_task_return_error (task, error);
-		g_object_unref (task);
+		g_simple_async_result_take_error (simple, error);
+		g_simple_async_result_complete_in_idle (simple);
+		g_object_unref (simple);
 		return;
 	}
 
-	g_task_run_in_thread (task, soup_multipart_input_stream_next_part_thread);
-	g_object_unref (task);
+	g_simple_async_result_run_in_thread (simple, soup_multipart_input_stream_next_part_thread,
+					     io_priority, cancellable);
+	g_object_unref (simple);
 }
 
 /**
  * soup_multipart_input_stream_next_part_finish:
  * @multipart: a #SoupMultipartInputStream.
  * @result: a #GAsyncResult.
- * @error: a #GError location to store any error, or %NULL to ignore.
+ * @error: a #GError location to store any error, or NULL to ignore.
  *
  * Finishes an asynchronous request for the next part.
  *
- * Return value: (nullable) (transfer full): a newly created
- * #GInputStream for reading the next part or %NULL if there are no
- * more parts.
+ * Return value: (transfer full): a newly created #GInputStream for
+ * reading the next part or %NULL if there are no more parts.
  *
  * Since: 2.40
  */
@@ -559,9 +565,22 @@ soup_multipart_input_stream_next_part_finish (SoupMultipartInputStream	*multipar
 					      GAsyncResult		*result,
 					      GError		       **error)
 {
-	g_return_val_if_fail (g_task_is_valid (result, multipart), FALSE);
+	GSimpleAsyncResult *simple;
+	GInputStream *new_stream;
 
-	return g_task_propagate_pointer (G_TASK (result), error);
+	g_return_val_if_fail (g_simple_async_result_is_valid (result, G_OBJECT (multipart),
+							      soup_multipart_input_stream_next_part_async), FALSE);
+
+	simple = G_SIMPLE_ASYNC_RESULT (result);
+
+	if (g_simple_async_result_propagate_error (simple, error))
+		return NULL;
+
+	new_stream = g_simple_async_result_get_op_res_gpointer (simple);
+	if (new_stream)
+		return g_object_ref (new_stream);
+
+	return NULL;
 }
 
 /**
@@ -578,9 +597,9 @@ soup_multipart_input_stream_next_part_finish (SoupMultipartInputStream	*multipar
  * Note that if a part had no headers at all an empty #SoupMessageHeaders
  * will be returned.
  *
- * Return value: (nullable) (transfer none): a #SoupMessageHeaders
- * containing the headers for the part currently being processed or
- * %NULL if the headers failed to parse.
+ * Return value: (transfer none): a #SoupMessageHeaders containing the headers
+ * for the part currently being processed or %NULL if the headers failed to
+ * parse.
  *
  * Since: 2.40
  */
